#!/bin/sh
#********************************************************************************
# (c) Copyright [2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
# are as may be set forth in the express warranty statements accompanying such products and services. Nothing
# herein should be construed as constituting an additional warranty. Micro Focus shall not be liable for technical
# or editorial errors or omissions contained herein. The information contained herein is subject to change without
# notice.
#********************************************************************************

# Certificate files and directories
CERT_DIR="/etc/opt/novell/iprintmobile/conf/certs"
RENDERER_CERT_PFX=$CERT_DIR/iprint_renderer_cert.pfx
ROOT_CA_CRT="/etc/opt/novell/certs/SSCert.pem"
KEYSTORE_FILE_PATH="/etc/opt/novell/iprintmobile/certs/keystore"

ERR_LOG="/var/opt/novell/log/iprintmobile/certman_oes.log"
touch $ERR_LOG

script_filename=$0

OPENSSL="/usr/bin/openssl"
HOSTNAME=$(hostname -f)

# Set validity for 2 Years 
VALIDITY_DAYS=$((365*2))
KEY_STRENGTH=2048

DOMAIN=$(hostname -d)
# for wild card certificate
CN="*.${DOMAIN}"

KEY_PASSWORD="changeit"

OPENSSL="/usr/bin/openssl"
KEYTOOL="/usr/bin/keytool"

PATH_TO_JAVA_KEYSTORE=$EXISTING_KEYSTORE

#Bundle Files for remote renderer installation
BUNDLE_FILE="/var/opt/novell/iprint/htdocs/iPrintRendererBundle.zip"
MSI_FILE="/var/opt/novell/iprint_downloads/WinRenderer/iPrintRendererSetup.msi"
SETUP_FILE="/var/opt/novell/iprint_downloads/WinRenderer/setup.exe"
README_FILE="/var/opt/novell/iprint_downloads/WinRenderer/ReadMe.txt"

logInfo()
{
	printf "\nINFO : ${@}" >> $ERR_LOG
}

createCSR()
{
	logInfo "Generating private key"
	$OPENSSL genrsa -out $CERT_DIR/iprint_renderer_cert.key $KEY_STRENGTH
	logInfo "Generated private key at $CERT_DIR/$HOSTNAME.key"
	$OPENSSL req -new -key $CERT_DIR/iprint_renderer_cert.key -subj "/CN=$CN"  -out $CERT_DIR/iprint_renderer_cert.csr
	logInfo "Generated CSR at $CERT_DIR/iprint_renderer_cert.csr"
	logInfo "Verifying certificate signing request.."
	$OPENSSL req -in $CERT_DIR/iprint_renderer_cert.csr -verify >> /dev/null
	res=$?
	if [[ $res -eq 1 ]]; then
		logInfo "Invalid CSR signature or CSR verification failed"
		echo "Invalid CSR signature or CSR verification failed"
	else
		logInfo "CSR generated successfully"
		echo "CSR file generated successfully: $CERT_DIR/iprint_renderer_cert.csr"
	fi
}

verifyCSR()
{
	logInfo "Verifying the generated csr.."
	$OPENSSL req -in $CERT_DIR/iprint_renderer_cert.csr -verify
	res=$?
	if [[ $res -eq 1 ]]; then
		logSevere "Invalid CSR signature or CSR verification failed"
	else
		logInfo "CSR is valid"
	fi
}

generatePFX()
{
	export PASSWORD=$KEY_PASSWORD
	logInfo "Converting der to pem certificate.."
	$OPENSSL x509 -inform der -in $CERT_DIR/iprint_renderer_cert.der -out $CERT_DIR/iprint_renderer_cert.pem
	
	logInfo "Converting to pfx.."
	$OPENSSL pkcs12 -export -out $RENDERER_CERT_PFX -inkey $CERT_DIR/iprint_renderer_cert.key -in $CERT_DIR/iprint_renderer_cert.pem -certfile $ROOT_CA_CRT -passin env:PASSWORD -passout env:PASSWORD
}

#Bundle Files for remote renderer installation
createBundle() 
{

	# Create bundle
	logInfo "Creating a bundle file for installing a Remote Renderer"
	if [ -f $RENDERER_CERT_PFX ]; then
		zip -j $BUNDLE_FILE $MSI_FILE  $RENDERER_CERT_PFX  $SETUP_FILE $README_FILE
	else
		zip -j $BUNDLE_FILE $MSI_FILE  $SETUP_FILE $README_FILE
	fi
	
	if [ $? != 0 ]; then
        logInfo "Failed to create a Remote Renderer bundle"
        exit 3
    else
        logInfo "Remote Renderer bundle created successfully"
    fi
}

updateKeystore()
{
	ALIAS="org_ca"
	$KEYTOOL -list -v -keystore $KEYSTORE_FILE_PATH -alias $ALIAS -storePass $KEY_PASSWORD -noprompt >> /dev/null
	if  [ $? == 0 ]; then
        logInfo "Key with alias '$ALIAS' exists. deleting it"
        $KEYTOOL -delete -alias $ALIAS -keystore $KEYSTORE_FILE_PATH -storePass $KEY_PASSWORD -noprompt
	fi
	
	$KEYTOOL -import -trustcacerts -alias $ALIAS -file $ROOT_CA_CRT -keystore $KEYSTORE_FILE_PATH -storepass $KEY_PASSWORD -noprompt
	if [ $? != 0 ]; then
        logInfo "Failed to import root certificate into java keystore"
        exit 3
    else
        logInfo "root certificate imported into java keystore successfully"
    fi
}

usage() 
{
    echo "Usage:"
    echo "sh ${script_filename} <option(s)>"
    echo "valid options: "
    echo "-c : Create a Certificate Signing Request (CSR)"
    echo "-g : Generate .pfx file and assign it to a Remote Renderer zip bundle"
	echo "-b : Create a Remote Renderer bundle with the updated files"
    echo "-u : Update the keystore file with the root certificate"
    exit 0
}

HELP="no"
GENERATE_CSR="no"
CONFIG_PFX_RENDERER="no"
UPDATE_KEYSTORE="no"
BUNDLE_RENDERER="no"

parseArgs()
{
	while getopts :hcgub o;
	do		
		case "$o" in
			h) HELP="yes";;
			c) GENERATE_CSR="yes";;
			g) CONFIG_PFX_RENDERER="yes";;
			u) UPDATE_KEYSTORE="yes";;
			b) BUNDLE_RENDERER="yes";;
			# Skip any wrong option. 
			# This might be some operation specific argument
			\?) ;;
		esac
	done

	if [ "${HELP}" == "yes" ]; then
		usage
	elif [ "${GENERATE_CSR}" == "yes" ]; then
		createCSR
	elif [ "${CONFIG_PFX_RENDERER}" == "yes" ]; then
		generatePFX
		createBundle
	elif [ "${UPDATE_KEYSTORE}" == "yes" ]; then
		updateKeystore
	elif [ "${BUNDLE_RENDERER}" == "yes" ]; then
		createBundle
	else
		usage
	fi
}

parseArgs $@
